@@ -10,7 +10,6 @@ use std::time::Duration;
 use ureq::{Agent, AgentBuilder, OrAnyStatus};
 use uuid::Uuid;
 const BASE: &str = "/_matrix/client/v3/";
-//const TIME_TO_IDLE: Duration = Duration::from_secs(3 * 60);
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
 pub struct StorageData {
     pub auth_token: String,
@@ -67,15 +66,10 @@ pub struct EventResponse {
 pub struct Client {
     agent: Agent,
     pub me: UserId,
-    //server: String,
     since: Option<String>,
     host: String,
-    //auth_token: String,
-    //pub joined_rooms: Vec<RoomId>,
     pub data: StorageData,
-    //username: String,
     room_list_updated: bool,
-    //starting: bool,
 }
 
 impl Client {
@@ -85,21 +79,15 @@ impl Client {
         server_name: impl Into<String>,
     ) -> Self {
         Self {
-            //auth_token: String::new(),
             host: host.into().trim_end_matches("/").to_string(),
             me: UserId::try_from(format!("@{}:{}", username.into(), server_name.into()).as_str())
                 .unwrap(),
             agent: AgentBuilder::new()
                 .timeout_read(Duration::from_secs(3))
                 .build(),
-            //joined_rooms: Vec::new(),
             since: None,
             data: StorageData::default(),
-            //username: username.into(),
-            //server: server_name.into(),
-            //last_event: EventResponse::default(),
             room_list_updated: false,
-            //starting: true,
         }
     }
     pub fn room_list_updated(&mut self) -> bool {
@@ -124,14 +112,12 @@ impl Client {
             if let Ok(z) =
                 ureq::get(&format!("{}/.well-known/matrix/client", self.host.clone())).call()
             {
-                //panic!("{}", z.into_string().unwrap());
                 let resp: Value = z.into_json().unwrap();
                 let base_url: String = resp["m.homeserver"]["base_url"]
                     .as_str()
                     .unwrap()
                     .to_string();
                 self.host = base_url.trim_end_matches("/").to_string();
-                //self.me = UserId::try_from(format!("@{}:{}", self.me.localpart(), self.host.clone().split_once("://").unwrap().1).as_str()).unwrap();
             }
         }
         self.data.auth_token = auth_token;
@@ -151,14 +137,12 @@ impl Client {
             if let Ok(z) =
                 ureq::get(&format!("{}/.well-known/matrix/client", self.host.clone())).call()
             {
-                //panic!("{}", z.into_string().unwrap());
                 let resp: Value = z.into_json().unwrap();
                 let base_url: String = resp["m.homeserver"]["base_url"]
                     .as_str()
                     .unwrap()
                     .to_string();
                 self.host = base_url.trim_end_matches("/").to_string();
-                //self.me = UserId::try_from(format!("@{}:{}", self.me.localpart(), self.host.clone().split_once("://").unwrap().1).as_str()).unwrap();
             }
         }
         let content = json!({
@@ -298,8 +282,6 @@ impl Client {
         resp
     }
     pub fn join_room_id(&self, room_id: ruma::RoomId) -> Value {
-        //let room_string = room_id_or_alias.into();
-        //let serv_name = server_name.into();
         let rid = room_id.to_string();
         let room = urlencoding::encode(rid.as_str());
         let content = json!({});
@@ -309,7 +291,6 @@ impl Client {
             .unwrap()
     }
     pub fn sync(&mut self) -> Result<EventResponse, Value> {
-        //let mut r = self.get("sync")
         let req: ureq::Response;
         if let Some(start) = self.since.clone() {
             req = self
@@ -342,9 +323,6 @@ impl Client {
                 .unwrap();
         }
         let val: Value = req.into_json().unwrap();
-        //self.get_tx().send(format!("{:#?}", val.to_string())).unwrap();
-        //sm_println!(self, "{:#?}", val);
-        //println!("{:#?}", val);
         if val.clone().as_object().unwrap().contains_key("error") {
             Err(val)
         } else {
@@ -358,8 +336,7 @@ impl Client {
         }
     }
     pub fn join_with_alias(&self, alias: impl Into<String>) {
-        //self.post(format!("join/{}", urlencoding::encode(alias.into())), content, None).unwrap().into_json().unwrap()
-        let room_alias = alias.into(); //format!("{}", alias.into());
+        let room_alias = alias.into();
         let this_room_id_val: Value = self
             .get(
                 format!(
@@ -375,7 +352,6 @@ impl Client {
             .unwrap();
         let room_id =
             ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap();
-        //self.data.room_to_aliases.insert(room_id.clone(), vec![room_alias.try_into().unwrap()]);
         self.join_room_id(room_id);
     }
     pub fn leave_room_id(&mut self, room_id: RoomId) -> Value {
@@ -388,12 +364,11 @@ impl Client {
             .into_json()
             .unwrap();
         self.data.joined_rooms.retain(|x| x != &room_id);
-        //self.data.room_to_name.remove(&room_id);
         self.room_list_updated = true;
         output
     }
     pub fn room_id_from_alias(&self, alias: impl Into<String>) -> RoomId {
-        let room_alias = alias.into(); //format!("{}", alias.into());
+        let room_alias = alias.into();
         let this_room_id_val: Value = self
             .get(
                 format!(
@@ -409,17 +384,6 @@ impl Client {
             .unwrap();
         ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap()
     }
-    /*
-    #[cfg(not(target_os = "android"))]
-    pub fn start(&mut self, password: impl Into<String>, room_ids: Vec<impl Into<String>>) {
-        let pass = password.into();
-        for this_room in room_ids {
-            let this_room_string = this_room.into();
-            let this_room_id_val: Value = self.get(format!("directory/room/{}", urlencoding::encode(this_room_string.as_str()).to_string().as_str()), None).unwrap().into_json().unwrap();
-            let room_id = ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap();
-        }
-    }
-    */
     pub fn send_message(&self, room_id: RoomId, message: impl Into<String>) {
         let msg = message.into();
         let txid = Uuid::new_v4()
@@ -436,20 +400,15 @@ impl Client {
 
     pub fn extract_data(&mut self, sync_results: EventResponse) -> Vec<MessageInfo> {
         let mut messages: Vec<MessageInfo> = Vec::new();
-        //if let RoomsmTypeData::Joined(ref x) = event_args.room_data {
         let x = sync_results.rooms.join;
         for (room_id, joined_room) in x.iter() {
-            //if event_args.ctrlc_handler.should_continue() {
-            //instructions.push(Instructions::AddRoom(room_id.clone()));
             if !self.data.joined_rooms.contains(room_id) {
                 self.data.joined_rooms.push(room_id.clone());
-                //self.data.room_to_name.insert(room_id.clone(), self.room_id_to_alias[room_id.clone()]);
                 self.room_list_updated = true;
             }
             let mut room_members: Vec<String> = Vec::new();
             let mut is_direct = false;
             for event in joined_room.state.events.clone() {
-                //let ev = event.deserialize().unwrap();
                 if let Ok(ev) = event.deserialize() {
                     if let ruma::events::AnySyncStateEvent::RoomAliases(room_aliases_event) =
                         ev.clone()
@@ -512,7 +471,6 @@ impl Client {
                     .room_to_direct_name
                     .insert(room_id.clone(), room_members[0].clone());
             }
-            //if !event_args.starting {
             for message_info in joined_room.timeline.events.iter().filter_map(|m| {
                 if let Ok(ruma::events::AnySyncRoomEvent::Message(message_event)) =
                     m.deserialize_as()
@@ -534,44 +492,35 @@ impl Client {
                         if sender == self.me {
                             return None;
                         }
-
+                        let message: String;
+                        let is_emote: bool;
                         if let room::message::MessageType::Text(text_message_event_content) =
                             room_message.content.msgtype
                         {
-                            let message = text_message_event_content.body;
-                            //event_args.tx.send(format!("<{}> {}", sender, message)).unwrap();
-                            //if message.starts_with(&event_args.prefix) {
-                            /*
-                            let words: Vec<String> = message.split_whitespace().map(|w| w.to_string()).collect();
-                            let args = words[1..].to_vec();
-                            let mut room_aliases: Vec<RoomAliasId> = Vec::new();
-                            if event_args.room_to_aliases.contains_key(&room_id) {
-                                room_aliases.append(&mut event_args.room_to_aliases[&room_id].clone());
-                            }
-                            */
-                            return Some(MessageInfo {
-                                message: message.clone(),
-                                //words: words,
-                                //args: args,
-                                sender: sender,
-                                room_id: room_id.clone(),
-                                //room_aliases: room_aliases,
-                            });
-                            //}
+                            message = text_message_event_content.body;
+                            is_emote = false;
+                        } else if let room::message::MessageType::Emote(
+                            emote_message_event_content,
+                        ) = room_message.content.msgtype
+                        {
+                            message = emote_message_event_content.body;
+                            is_emote = true;
+                        } else {
+                            return None;
                         }
+                        return Some(MessageInfo {
+                            message: message.clone(),
+                            sender: sender,
+                            room_id: room_id.clone(),
+                            is_emote: is_emote,
+                        });
                     }
                 }
                 None
             }) {
                 messages.push(message_info);
             }
-            //}
-            //} //else {
-            //instructions.push(Instructions::Quit(event_args.cleanup_on_ctrlc));
-            //return instructions;
-            //}
         }
-        //}
         messages
     }
 }
@@ -580,4 +529,5 @@ pub struct MessageInfo {
     pub room_id: RoomId,
     pub sender: UserId,
     pub message: String,
+    pub is_emote: bool,
 }
